<?php
/**
 * functions.php
 *
 * The theme's functions and definitions.
 */

/**
 * ----------------------------------------------------------------------------------------
 * 1.0 - Define constants.
 * ----------------------------------------------------------------------------------------
 */
define( 'THEMEROOT', get_stylesheet_directory_uri() );
define( 'IMAGES', THEMEROOT . '/images' );
define( 'SCRIPTS', THEMEROOT . '/js' );
require_once( 'wp_bootstrap_navwalker.php' );


/**
 * ----------------------------------------------------------------------------------------
 * 4.0 - Set up theme default and register various supported features.
 * ----------------------------------------------------------------------------------------
 */

if ( ! function_exists( 'alpha_setup' ) ) {
	function alpha_setup() {
		/**
		 * Make the theme available for translation.
		 */
		$lang_dir = THEMEROOT . '/languages';
		load_theme_textdomain( 'alpha', $lang_dir );

		/**
		 * Add support for post formats.
		 */
		add_theme_support( 'post-formats',
			array(
				'gallery',
				'link',
				'image',
				'quote',
				'video',
				'audio'
			)
		);

		/**
		 * Add support for automatic feed links.
		 */
		add_theme_support( 'automatic-feed-links' );
		add_theme_support( 'widgets' );

		/**
		 * Add support for post thumbnails.
		 */
		add_theme_support( 'post-thumbnails' );


		/**
		 * Register nav menus.
		 */
		register_nav_menus(
			array(
				'main-menu' => __( 'القائمة الرئيسية', 'alpha' )
			)
		);
	}

	add_action( 'after_setup_theme', 'alpha_setup' );
}


/**
 * ----------------------------------------------------------------------------------------
 * 8.0 - Function that validates a field's length.
 * ----------------------------------------------------------------------------------------
 */
if ( ! function_exists( 'alpha_validate_length' ) ) {
	function alpha_validate_length( $fieldValue, $minLength ) {
		// First, remove trailing and leading whitespace
		return ( strlen( trim( $fieldValue ) ) > $minLength );
	}
}


/**
 * ----------------------------------------------------------------------------------------
 * 10.0 - Load the custom scripts for the theme.
 * ----------------------------------------------------------------------------------------
 */
if ( ! function_exists( 'alpha_scripts' ) ) {
	function alpha_scripts() {
		// Adds support for pages with threaded comments
		if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
			wp_enqueue_script( 'comment-reply' );
		}

		// Register scripts
		wp_register_script( 'bootstrap-js', SCRIPTS . '/bootstrap.min.js', array( 'jquery' ), false, true );
		wp_register_script( 'classie', SCRIPTS . '/classie.js', array( 'jquery' ), false, true );
//		wp_register_script( 'hoverdir', SCRIPTS . '/jquery.hoverdir.js', array( 'jquery' ), false, true );
		wp_register_script( 'initslider-1', THEMEROOT . '/sliderengine/initslider-1.js', array( 'jquery' ), false, true );
		wp_register_script( 'ofroogaloop2', THEMEROOT . '/sliderengine/froogaloop2.min.js', array( 'jquery' ), false, true );
		wp_register_script( 'amazingslider', THEMEROOT . '/sliderengine/amazingslider.js', array( 'jquery' ), false, true );


		// Load the custom scripts
		wp_enqueue_script( 'bootstrap-js' );
		wp_enqueue_script( 'classie' );
//		wp_enqueue_script( 'hoverdir' );
		wp_enqueue_script( 'initslider-1' );
		wp_enqueue_script( 'ofroogaloop2' );
		wp_enqueue_script( 'amazingslider' );


		wp_register_script( 'custom', SCRIPTS . '/custom.js', array( 'jquery' ), false, true );
		wp_enqueue_script( 'custom' );


		// Load the stylesheets
		wp_enqueue_style( 'bootstrap', THEMEROOT . '/css/bootstrap.min.css' );
		wp_enqueue_style( 'Fontawesome', THEMEROOT . '/css/font-awesome.min.css' );
		wp_enqueue_style( 'animate', THEMEROOT . '/css/animate.css' );
		wp_enqueue_style( 'owl.carousel', THEMEROOT . '/css/owl.carousel.css' );
		wp_enqueue_style( 'amazingslider-1', THEMEROOT, '/sliderengine/amazingslider-1.css' );

		if ( is_rtl() ) {

			wp_enqueue_style( 'bootstrap-rtl', THEMEROOT . '/css/bootstrap-rtl.min.css' );
			wp_enqueue_style( 'vertical-tabs', THEMEROOT . '/css/bootstrap.vertical-tabs.min.css' );
			wp_enqueue_style( 'rtl', THEMEROOT . '/style.css' );

		} else {
			wp_enqueue_style( 'vertical-tabs', THEMEROOT . '/css/bootstrap.vertical-tabs.min.css' );
			wp_enqueue_style( 'en', THEMEROOT . '/en.css' );

		}

	}

	add_action( 'wp_enqueue_scripts', 'alpha_scripts' );
}


/**
 * ----------------------------------------------------------------------------------------
 * 11.0 - Custom Post Type.
 * ----------------------------------------------------------------------------------------
 */

register_post_type( 'slider', array(
	'public'              => true,
	'hierarchical'        => false,
	'exclude_from_search' => false,
	'has_archive'         => false,
	'rewrite'             => array( 'slug' => 'slider' ),
	'capability_type'     => 'post',
	'taxonomies'          => array( 'post_tag' ),
	'labels'              => array(
		'name'               => 'السلايدر',
		'add_new_item'       => 'إضافة سلايدر',
		'singular_name'      => 'السلايدر',
		'menu_name'          => 'السلايدر',
		'name_admin_bar'     => 'سلايدر',
		'all_items'          => 'كافة السلايدر',
		'add_new'            => 'إضافة سلايدر',
		'edit_item'          => 'تحرير السلايدر',
		'new_item'           => 'سلايدر جديدة',
		'view_item'          => 'مشاهدة السلايدر',
		'search_items'       => 'البحث عن سلايدر',
		'not_found'          => 'لا يوجد سلايدرات.',
		'not_found_in_trash' => 'لا يوجد سلايدرات',
		'parent_item_colon'  => 'القسم الرئيسى'
	),
	'supports'            => array( 'title', 'thumbnail', 'editor' )
) );

register_post_type( 'shop', array(
	'public'              => true,
	'hierarchical'        => false,
	'exclude_from_search' => false,
	'has_archive'         => false,
	'rewrite'             => array( 'slug' => 'shop' ),
	'capability_type'     => 'post',
	'taxonomies'          => array( 'post_tag' ),
	'labels'              => array(
		'name'               => 'المنتجات',
		'add_new_item'       => 'إضافة منتج',
		'singular_name'      => 'المنتجات',
		'menu_name'          => ' المنتجات',
		'name_admin_bar'     => 'منتج',
		'all_items'          => 'كافة المنتجات',
		'add_new'            => 'إضافة منتج',
		'edit_item'          => 'تحرير المنتج',
		'new_item'           => 'منتج جديد',
		'view_item'          => 'مشاهدة المنتج',
		'search_items'       => 'البحث عن منتج',
		'not_found'          => 'لا يوجد منتجات.',
		'not_found_in_trash' => 'لا يوجد منتجات',
		'parent_item_colon'  => 'القسم الرئيسى'
	),
	'supports'            => array( 'title', 'thumbnail', 'editor', 'comments' )
) );

/*************************************************/
/* Register Taxonami */
/*************************************************/
add_action( 'init', 'create_products' );

function create_products() {
	register_taxonomy(
		'products',
		'shop',
		array(
			'label'             => __( 'أقسام المنتجات' ),
			'hierarchical'      => true,
			'show_ui'           => true,
			'show_admin_column' => true,
			'show_in_nav_menus' => true,
			'show_tagcloud'     => true,
			'public'            => true
		)
	);
}


/*************************************************/
/* Sizes */
/*************************************************/

add_image_size( 'slider-image', 1600, 600, true );
add_image_size( 'news-img', 263, 166, true );
add_image_size( 'activities-img', 146, 97, true );
add_image_size( 'activities-inside-img', 200, 200, true );
add_image_size( 'doc-img', 170, 170, true );


/*************************************************/
/* Page Option */
/*************************************************/

if ( function_exists( 'acf_add_options_page' ) ) {

	acf_add_options_page( array(
		'page_title' => 'اعدادات القالب',
		'menu_title' => 'اعدادات القالب',
		'menu_slug'  => 'theme-general-settings',
		'capability' => 'edit_posts',
		'redirect'   => false
	) );

}

